YayMounts = LibStub("AceAddon-3.0"):NewAddon("YayMounts","AceConsole-3.0", "AceEvent-3.0")
AceGUI = LibStub("AceGUI-3.0")

local function size(t)
  local i = 0
  for k,v in pairs(t) do
    i = i + 1
  end
  return i
end

local function pairsByKeys (t, f)
  local a = {}
    for n in pairs(t) do 
      table.insert(a, n) 
    end
    table.sort(a, f)
    local i = 0      -- iterator variable
    local iter = function ()   -- iterator function
      i = i + 1
      if a[i] == nil then 
        return nil
      else 
        return a[i], t[a[i]]
      end
    end
  return iter
end

local options = {
  name="YayMounts",
  handler = YayMounts,
  type = "group",
  args = {
    enable = {
      type = "execute",
      name = YayMountsLocale.Misc.optionsGUIButton,
      func = "Frames",
    },
  }
}

local version = "1.7.8"
local t, p, tProfiles, tProfiles2, newMounts, aspectTable = {}, {}, {}, {}, {}, {13165, 5118, 13159}
local playerFaction, usePassengerMount, forceRepairMount = nil, false, false
local currentTab, currentSubTab
local localClass, class = UnitClass("player")
local f = CreateFrame("frame")
local f2 = CreateFrame("frame")

local defaults = { 
  global = { 
    ["CV"] = 1.0, --Current version
  },
  profile = {
    settings = {
      ["ER"] = true, --emptyrandom: use random mount if none selected
      ["FFP"] = false, --Druid: Use flight form with priority
      ["TTT"] = false, --Travel to Travel, no human form. Mostly for pvp.
      ["MTFF"] = false, --Druid: When flying mounted shift directly to flight form
      ["FBP"] = false, --Prioritize flying broom, even over Flight form
      ["HM"] = false, -- Has Mount
      ["NDWF"] = false, -- No Dismount while Flying. Safe!
      ["FFM"] = true, --Force Flying Mount (in ground areas)
      ["MR"] = true, --Monk: Roll
      ["RS"] = true, --Rogue: Sprint
      ["PMLS"] = true, --Priest/Mage: Levitate/Slowfall
      ["URM"] = true, --use a repair mount if one exists
	  ["TELAARI"] = true, --Use the Telaari Talbuk (or horde equivalent) in Nagrand
    },
    misc = {
      ["DBSize"] = 0, -- Database size
      ["MinDurability"] = 0.2, --summon repair mount if under X durability
      
      RepairMounts = {},
      NumMounts = {
        ["Total"] = 0,
        ["Ground"] = 0,
        ["Flying"] = 0,
		["Profession"] = 0,
        ["Special"] = 0,
        },
    },    
    mounts = {
      ['*'] = {
      },
    },
  }
}

function YayMounts:OnInitialize() 
  self.db = LibStub("AceDB-3.0"):New("YayMountsDB", defaults, "Default")
  local ymOptions = LibStub("AceConfigRegistry-3.0")
  ymOptions:RegisterOptionsTable("YayMounts",options)
  self.ymOpFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("YayMounts","YayMounts")
  self:RegisterChatCommand("ym", "ChatCommand")
  self:RegisterChatCommand("yaymounts", "ChatCommand")
end

function YayMounts:ChatCommand(input) --Chat command function
  if input == "reload" or input == "rl" then
    YayMounts:ReSort()
    self:Print(YayMountsLocale.Misc.mountsReloaded)
  
  else
    YayMounts:Frames("tab1")
  end
end

function YayMounts:OnEnable()-- addon enabled
	YayMounts:AlphaSortProfileDB()

	local _, race = UnitRace("player")
    
	--if self.db.global["CV"] ~= version then
		--self.db.global["CV"] = version
		YayMounts:ReSort()
	--end

	YayMounts:MapOverrides()
  
	-- Register events
	self:RegisterEvent("UPDATE_BINDINGS", "MapOverrides")
	self:RegisterEvent("COMPANION_LEARNED", "EventHandler")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "EventHandler")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EventHandler")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "EventHandler")

	--if race == "Worgen" and UnitLevel("player") < 21 then
		--self:RegisterEvent("PLAYER_LEVEL_UP", "EventHandler")
	--end
end
--------------------------------------------------------------
--database handling and creation functions
--------------------------------------------------------------
function YayMounts:IsUsableMount(index)
	local n,spellID,_,_,isUsable,_,_,_,_,isHidden,isCollected = C_MountJournal.GetMountInfo(index)
	return isUsable
end

function YayMounts:AlphaSortProfileDB()
  table.wipe(tProfiles)
  table.wipe(tProfiles2)
  
  self.db:GetProfiles(tProfiles)

  for k, v in pairs(tProfiles) do
    tProfiles2[v] = k     --put names as key, order #'s as value
  end
  
  table.wipe(tProfiles)
  
  for k, v in pairsByKeys(tProfiles2) do
    tinsert(tProfiles, k)   --sort alphabetically, name as value
  end
  
end

function YayMounts:ReSort()
  local tempTable = {}
  
  for k,v in pairs(self.db.profile.mounts) do
    tempTable[k] = self.db.profile.mounts[k][1]
  end
  table.wipe(self.db.profile.mounts)
  
  YayMounts:ReloadMounts(tempTable)
end

function YayMounts:ProfessionCheck(spellID)
	if YayMountsType["Professions"][spellID] then
		local prof1, prof2, _,_,_,_ = GetProfessions()
		local SL1, SL2

		if prof1 then
			prof1, _, SL1 = GetProfessionInfo(prof1)

			if YayMountsType["Professions"][spellID][1] == prof1 then
				if YayMountsType["Professions"][spellID][2] <= SL1 then
					return true
				end
			end
		end

		if prof2 then
			prof2, _, SL2 = GetProfessionInfo(prof2)

			if YayMountsType["Professions"][spellID][1] == prof2 then
				if YayMountsType["Professions"][spellID][2] <= SL2 then 
				  return true
				end
			end
		end

		return false
	end

	return true
end

function YayMounts:ReputationCheck(spellID)
  if YayMountsType["Reputation"][spellID] then    
	-- Nothing gated behind reputation right now
    return false
  end
  
  return true -- Not a known reputation restricted spell
end

function YayMounts:SpellToName(spell) --converts spell ID to Name
  local name = GetSpellInfo(spell)
  return name
end

function YayMounts:ItemToName(itemID) --converts spell ID to Name
  local name = GetItemInfo(itemID)
  return name
end

function YayMounts:GetDbValue(info, key)
  if info == "mounts" then
    return self.db.profile.mounts[key][1]
  
  elseif info == "settings" then
    return self.db.profile.settings[key]
    
  elseif info == "misc" then
    return self.db.profile.misc[key]
  end
end

function YayMounts:SetDbValue(info, key, state)
  if info == "mounts" then
    self.db.profile.mounts[key][1] = state
  elseif info == "misc" then
    self.db.profile.misc[key] = state  
  elseif info == "settings" then
    self.db.profile.settings[key] = state
  end
end

function YayMounts:ClearMounts(mountType)-- clear all mounts
	for spellID,v in pairs(self.db.profile.mounts) do
		if self.db.profile.mounts[spellID][2] == mountType and t[spellID] ~= nil then
			if YayMounts:ProfessionCheck(spellID) then
				self.db.profile.mounts[spellID][1] = false
				t[spellID]:SetValue(false)
			end
		end
	end
end

function YayMounts:SelectAllMounts(mountType)-- select all mounts
	for spellID,v in pairs(self.db.profile.mounts) do
		if self.db.profile.mounts[spellID][2] == mountType and t[spellID] ~= nil then
			if YayMounts:ProfessionCheck(spellID) then
				self.db.profile.mounts[spellID][1] = true
				t[spellID]:SetValue(true)
			end
		end
	end
end

function YayMounts:BuildList(mountType)--Builds and draws the checkboxes for the opened tab
	t = {}
	for index = 1, C_MountJournal.GetNumMounts(), 1 do
		local name,spellID,icon,_,_,_,_,_,faction,_,_ = C_MountJournal.GetMountInfo(index)
		if faction == nil or faction == playerFaction then
			if self.db.profile.mounts[spellID][2] == mountType then
				t[spellID] = AceGUI:Create("CheckBox")
				t[spellID]:SetImage(icon)
				t[spellID]:SetLabel(name)
				t[spellID]:SetValue(YayMounts:GetDbValue("mounts", spellID))
				t[spellID]:SetCallback("OnValueChanged",  
					function() 
						YayMounts:SetDbValue("mounts", spellID, t[spellID]:GetValue()) 
					end)
				YayMountsContent:AddChild(t[spellID])
			end
		end
	end
end

function YayMounts:ReloadMounts(tempTable)--Rebuilds the database. Activates on load
  self.db.profile.misc.NumMounts["Ground"] = 0
  self.db.profile.misc.NumMounts["Flying"] = 0
  self.db.profile.misc.NumMounts["Profession"] = 0
  self.db.profile.misc.NumMounts["Special"] = 0
  self.db.profile.misc.NumMounts["Total"] = 0
  
	for index = 1, C_MountJournal.GetNumMounts(), 1 do
		local n,spellID,_,_,_,isUsable,_,_,_,isHidden,isCollected = C_MountJournal.GetMountInfo(index)
		local _, _, _, _, mountTypeID = C_MountJournal.GetMountInfoExtra(index)
		
		-- Deal with class mounts not showing up as collected on other classes.
		if(YayMountsType["Class"][spellID] ~= nil) then
			if(tempTable[spellID] ~= nil) then
				isCollected = true
			end
		end
		
		if(isCollected and mountTypeID ~= 242) then
			local mountType = YayMountsType["Mounts"][mountTypeID]
			if(YayMountsType["Professions"][spellID] ~= nil) then
				mountType = "profession"
			end
		  
		  
			if(mountType ~= nil) then
				--Check if we have a repair mount
				if spellID == 61425 or spellID == 61447 or spellID == 122708 then
					self.db.profile.misc.RepairMounts[spellID] = true
				end

				if mountType == nil and spellID ~= 75207 then --Found a 'new' mount, ie unknown type
					self.db.profile.misc.NumMounts["Total"] = self.db.profile.misc.NumMounts["Total"] - 1
					newMounts[spellID] = name 
				else
					-- Keep old selection setting
					if tempTable[spellID] == nil then
						self.db.profile.mounts[spellID] = {true, mountType, index}
					else
						self.db.profile.mounts[spellID] = {tempTable[spellID], mountType, index}
					end
				end

				--Increment the counters
				if mountType == "ground" then
					self.db.profile.misc.NumMounts["Ground"] = self.db.profile.misc.NumMounts["Ground"] + 1
				elseif mountType == "fly" then
					self.db.profile.misc.NumMounts["Flying"] = self.db.profile.misc.NumMounts["Flying"] + 1
				elseif mountType == "special" then
					self.db.profile.misc.NumMounts["Special"] = self.db.profile.misc.NumMounts["Special"] + 1
				elseif mountType == "profession" then
					self.db.profile.misc.NumMounts["Profession"] = self.db.profile.misc.NumMounts["Profession"] + 1
				end

				self.db.profile.misc.NumMounts["Total"] = self.db.profile.misc.NumMounts["Total"] + 1
			end
		end
	end
  
	if self.db.profile.misc.NumMounts["Total"] > 0 then
		self.db.profile.settings["HM"] = true
	end
  
	--self:Print("Loaded", self.db.profile.misc.NumMounts["Total"], "mounts.")
end
--------------------------------------------------------------
--GUI functions for drawing frames
--------------------------------------------------------------
local function DrawGroup1a(container) --Ground Mounts, Tab1
  container:SetLayout("Flow")
    
  local selectallbutton = AceGUI:Create("Button")
  selectallbutton:SetText("Select All")
  selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("ground") end)
  container:AddChild(selectallbutton)
    
  local clearallbutton = AceGUI:Create("Button")
  clearallbutton:SetText("Clear All")
  clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("ground") end)
  container:AddChild(clearallbutton)
  
  local scrollcontainerframe = AceGUI:Create("SimpleGroup")
  scrollcontainerframe:SetLayout("Fill")
  scrollcontainerframe:SetFullWidth(true)
  scrollcontainerframe:SetFullHeight(true)
  container:AddChild(scrollcontainerframe)

  local scrollframe = AceGUI:Create("ScrollFrame")
  scrollcontainerframe:AddChild(scrollframe)
  
  
  YayMountsContent = AceGUI:Create("InlineGroup")
  YayMountsContent:SetFullWidth(true)
  YayMountsContent:SetLayout("Flow")
  
  YayMounts:BuildList("ground")
  scrollframe:AddChild(YayMountsContent)
end

local function DrawGroup2a(container) --Flying Mounts, Tab2
  container:SetLayout("Flow")
  
  local selectallbutton = AceGUI:Create("Button")
  selectallbutton:SetText("Select All")
  selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("fly") end)
  container:AddChild(selectallbutton)
    
  local clearallbutton = AceGUI:Create("Button")
  clearallbutton:SetText("Clear All")
  clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("fly") end)
  container:AddChild(clearallbutton)
  
  local scrollcontainerframe = AceGUI:Create("SimpleGroup")
  scrollcontainerframe:SetLayout("Fill")
  scrollcontainerframe:SetFullWidth(true)
  scrollcontainerframe:SetFullHeight(true)
  container:AddChild(scrollcontainerframe)

  local scrollframe = AceGUI:Create("ScrollFrame")
  scrollcontainerframe:AddChild(scrollframe)
  
  YayMountsContent = AceGUI:Create("InlineGroup")
  YayMountsContent:SetFullWidth(true)
  YayMountsContent:SetLayout("Flow")
  
  YayMounts:BuildList("fly")
  scrollframe:AddChild(YayMountsContent)
end

local function DrawGroup3a(container) --Profession Mounts, Tab3
	container:SetLayout("Flow")
	
	local selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", 
		function() 
			for spellID,v in pairs(YayMounts.db.profile.mounts) do
				if t[spellID] ~= nil and YayMountsType["Professions"][spellID] ~= nil then
					YayMounts.db.profile.mounts[spellID][1] = true
					t[spellID]:SetValue(true)
				end
			end
		end)
	container:AddChild(selectallbutton)

	local clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", 
		function() 
			for spellID,v in pairs(YayMounts.db.profile.mounts) do
				if t[spellID] ~= nil and YayMountsType["Professions"][spellID] ~= nil then
					YayMounts.db.profile.mounts[spellID][1] = false
					t[spellID]:SetValue(false)
				end
			end
		end)
	container:AddChild(clearallbutton)

	local scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	local scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)

	YayMountsContent = AceGUI:Create("InlineGroup")
	YayMountsContent:SetFullWidth(true)
	YayMountsContent:SetLayout("Flow")


	YayMounts:BuildList("profession")
	scrollframe:AddChild(YayMountsContent)
end

local function DrawGroup4a(container) --Special Mounts, Tab4
  container:SetLayout("Flow")
  
  local selectallbutton = AceGUI:Create("Button")
  selectallbutton:SetText("Select All")
  selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("special") end)
  container:AddChild(selectallbutton)
    
  local clearallbutton = AceGUI:Create("Button")
  clearallbutton:SetText("Clear All")
  clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("special") end)
  container:AddChild(clearallbutton)
    
  local scrollcontainerframe = AceGUI:Create("SimpleGroup")
  scrollcontainerframe:SetLayout("Fill")
  scrollcontainerframe:SetFullWidth(true)
  scrollcontainerframe:SetFullHeight(true)
  container:AddChild(scrollcontainerframe)
    
  local scrollframe = AceGUI:Create("ScrollFrame")
  scrollcontainerframe:AddChild(scrollframe)
    
  YayMountsContent = AceGUI:Create("InlineGroup")
  YayMountsContent:SetFullWidth(true)
  YayMountsContent:SetLayout("Flow")
    
  
  YayMounts:BuildList("special")
  scrollframe:AddChild(YayMountsContent)
end

local function SelectGroupMounts(container, event, group)
	container:ReleaseChildren()
	if group == "tab1" then
		DrawGroup1a(container)
	elseif group == "tab2" then
		DrawGroup2a(container)
	elseif group == "tab3" then
		DrawGroup3a(container)
	elseif group == "tab4" then
		DrawGroup4a(container)
	end
	currentSubTab = group
end

local function DrawGroup1(container)--MOUNTS
  container:SetLayout("Fill")

  local tab  = AceGUI:Create("TabGroup")
  tab:SetLayout("Flow")
  tab:SetTabs({{text="Ground ("..YayMounts.db.profile.misc.NumMounts["Ground"]..")", value="tab1"}, {text="Flying ("..YayMounts.db.profile.misc.NumMounts["Flying"]..")", value="tab2"}, {text="Profession ("..YayMounts.db.profile.misc.NumMounts["Profession"]..")", value="tab3"}, {text="Special ("..YayMounts.db.profile.misc.NumMounts["Special"]..")", value="tab4"}})
  tab:SetCallback("OnGroupSelected", SelectGroupMounts)
  
  if currentSubTab ~= nil then
    tab:SelectTab(currentSubTab)
  else
    tab:SelectTab("tab1")
  end
  container:AddChild(tab)
end

local function DrawGroup2(container)--MOUNT OPTIONS
	container:SetLayout("Flow")
  
	local yayMountsBindingButton = AceGUI:Create("Keybinding")
	local yayMountsBindingButton2 = AceGUI:Create("Keybinding")
	local yayMountsBindingButton3 = AceGUI:Create("Keybinding")
	local yayMountsBindingButton4 = AceGUI:Create("Keybinding")
	
	yayMountsBindingButton:SetLabel(BINDING_NAME_YAYMOUNTBINDING)
	yayMountsBindingButton:SetKey(GetBindingKey("YAYMOUNTBINDING"))
	yayMountsBindingButton:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton:GetKey(), 1) 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton)

	
	yayMountsBindingButton2:SetLabel(BINDING_NAME_YAYMOUNTBINDING2)
	yayMountsBindingButton2:SetKey(GetBindingKey("YAYMOUNTBINDING2"))
	yayMountsBindingButton2:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton2:GetKey(), 2); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton2)
  
	
	yayMountsBindingButton3:SetLabel(BINDING_NAME_YAYMOUNTBINDING3)
	yayMountsBindingButton3:SetKey(GetBindingKey("YAYMOUNTBINDING3"))
	yayMountsBindingButton3:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton3:GetKey(), 3); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton3)
	
	
	yayMountsBindingButton4:SetLabel(BINDING_NAME_YAYMOUNTBINDING4)
	yayMountsBindingButton4:SetKey(GetBindingKey("YAYMOUNTBINDING4"))
	yayMountsBindingButton4:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton4:GetKey(), 4); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton4)

	-- No dismount while flying
	local nDWF = AceGUI:Create("CheckBox")
	nDWF:SetLabel(YayMountsLocale.String.NDWF)
	nDWF:SetValue(YayMounts:GetDbValue("settings", "NDWF"))
	nDWF:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "NDWF", nDWF:GetValue()) end)
	nDWF:SetFullWidth(true)
	container:AddChild(nDWF)
	
	-- Use Telaari Talbuk (or horde version) in Nagrand
	local telaari = AceGUI:Create("CheckBox")
	telaari:SetLabel(YayMountsLocale.String.TELAARI)
	telaari:SetValue(YayMounts:GetDbValue("settings", "TELAARI"))
	telaari:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "TELAARI", telaari:GetValue()) end)
	telaari:SetFullWidth(true)
	container:AddChild(telaari)

	-- Emergency random mount if no mounts useable from selected mounts
	local emptyRandom = AceGUI:Create("CheckBox")
	emptyRandom:SetLabel(YayMountsLocale.String.ER)
	emptyRandom:SetValue(YayMounts:GetDbValue("settings", "ER"))
	emptyRandom:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "ER", emptyRandom:GetValue()) end)
	emptyRandom:SetFullWidth(true)
	container:AddChild(emptyRandom)

	local forceFlyingMount = AceGUI:Create("CheckBox")
	forceFlyingMount:SetLabel(YayMountsLocale.String.FFM)
	forceFlyingMount:SetValue(YayMounts:GetDbValue("settings", "FFM"))
	forceFlyingMount:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "FFM", forceFlyingMount:GetValue()) end)
	forceFlyingMount:SetFullWidth(true)
	container:AddChild(forceFlyingMount)

	local sliderRepairMount = AceGUI:Create("Slider") --repair mount slider in decimal %
	sliderRepairMount:SetLabel("Low Durability %")
	sliderRepairMount:SetValue(YayMounts:GetDbValue("misc", "MinDurability")*100)
	sliderRepairMount:SetCallback("OnMouseUp", function() YayMounts:SetDbValue("misc", "MinDurability", (sliderRepairMount:GetValue()/100)) end)
	sliderRepairMount:SetSliderValues(0, 100, 5)
	sliderRepairMount:SetDisabled( not YayMounts:GetDbValue("settings", "URM"))

	local repairMount = AceGUI:Create("CheckBox")
	repairMount:SetLabel(YayMountsLocale.String.URM)
	repairMount:SetValue(YayMounts:GetDbValue("settings", "URM"))
	repairMount:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "URM", repairMount:GetValue()) sliderRepairMount:SetDisabled(not repairMount:GetValue()) end)
	repairMount:SetFullWidth(true)
  
	container:AddChild(repairMount)
	container:AddChild(sliderRepairMount)
  
	if GetItemCount(37011, false) > 0 then
		local flyingBroomPriority = AceGUI:Create("CheckBox")
		flyingBroomPriority:SetLabel(YayMountsLocale.String.FBP)
		flyingBroomPriority:SetValue(YayMounts:GetDbValue("settings", "FBP"))
		flyingBroomPriority:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "FBP", flyingBroomPriority:GetValue()) end)
		flyingBroomPriority:SetFullWidth(true)
		container:AddChild(flyingBroomPriority)
	end
end

local function DrawGroup3(container) --CLASS OPTIONS
  container:SetLayout("Flow")
  
  if class == "MONK" then
    local monkRoll = AceGUI:Create("CheckBox")
    monkRoll:SetLabel(YayMountsLocale.String.MR)
    monkRoll:SetValue(YayMounts:GetDbValue("settings", "MR"))
    monkRoll:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "MR", monkRoll:GetValue()) end)
    monkRoll:SetFullWidth(true)
    container:AddChild(monkRoll)
  end
  
  if class == "ROGUE" then
    local rogueSprint = AceGUI:Create("CheckBox")
    rogueSprint:SetLabel(YayMountsLocale.String.RS)
    rogueSprint:SetValue(YayMounts:GetDbValue("settings", "RS"))
    rogueSprint:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "RS", rogueSprint:GetValue()) end)
    rogueSprint:SetFullWidth(true)
    container:AddChild(rogueSprint)
  end
  
  if class == "DRUID" then
    local flightFormPriority = AceGUI:Create("CheckBox")
    flightFormPriority:SetLabel(YayMountsLocale.String.FFP)
    flightFormPriority:SetValue(YayMounts:GetDbValue("settings", "FFP"))
    flightFormPriority:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "FFP", flightFormPriority:GetValue()) end)
    flightFormPriority:SetFullWidth(true)
    container:AddChild(flightFormPriority)
    
    local travelToTravel = AceGUI:Create("CheckBox")
    travelToTravel:SetLabel(YayMountsLocale.String.TTT)
    travelToTravel:SetValue(YayMounts:GetDbValue("settings", "TTT"))
    travelToTravel:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "TTT", travelToTravel:GetValue()) end)
    travelToTravel:SetFullWidth(true)
    container:AddChild(travelToTravel)
    
    local mountToFlightForm = AceGUI:Create("CheckBox")
    mountToFlightForm:SetLabel(YayMountsLocale.String.MTFF)
    mountToFlightForm:SetValue(YayMounts:GetDbValue("settings", "MTFF"))
    mountToFlightForm:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "MTFF", mountToFlightForm:GetValue()) end)
    mountToFlightForm:SetFullWidth(true)
    container:AddChild(mountToFlightForm)
  end
  
  if class == "MAGE" then
    local pMLS = AceGUI:Create("CheckBox")
    pMLS:SetLabel(YayMountsLocale.String.MS)
    pMLS:SetValue(YayMounts:GetDbValue("settings", "PMLS"))
    pMLS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "PMLS", pMLS:GetValue()) end)
    pMLS:SetFullWidth(true)
    container:AddChild(pMLS)
  end
  
  if class == "PRIEST" then
    local pMLS = AceGUI:Create("CheckBox")
    pMLS:SetLabel(YayMountsLocale.String.PL)
    pMLS:SetValue(YayMounts:GetDbValue("settings", "PMLS"))
    pMLS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "PMLS", pMLS:GetValue()) end)
    pMLS:SetFullWidth(true)
    container:AddChild(pMLS)
  end
end

local function DrawGroup4(container) --PROFILE OPTIONS
  container:SetLayout("List")
  local cPV, tempStr = 1, YayMounts.db:GetCurrentProfile()
  
  for k,v in pairs(tProfiles) do
    if v == tempStr then
      cPV = k
    end
  end
  
  local createProfile = AceGUI:Create("EditBox")
  createProfile:SetLabel("Create new Profile")
  createProfile:SetMaxLetters(15) 
  
  local currentProfile = AceGUI:Create("Dropdown")
  currentProfile:SetList(tProfiles)
  currentProfile:SetLabel("Current Profile:")
  currentProfile:SetValue(cPV)
  
  local copyFrom = AceGUI:Create("Dropdown")
  copyFrom:SetList(tProfiles)
  copyFrom:SetItemDisabled(cPV, true)
  copyFrom:SetLabel("Copy settings from:")

  
  local copyButton = AceGUI:Create("Button")
  copyButton:SetWidth(75)
  copyButton:SetText("Copy")
  copyButton:SetDisabled(true)
  
  local deleteSelect = AceGUI:Create("Dropdown")
  deleteSelect:SetList(tProfiles)
  deleteSelect:SetItemDisabled(cPV, true)
  deleteSelect:SetLabel("Delete profile:")
  
  local deleteButton = AceGUI:Create("Button")
  deleteButton:SetWidth(100)
  deleteButton:SetText("Delete")
  deleteButton:SetDisabled(true)
  
  --Callbacks
  createProfile:SetCallback("OnEnterPressed", 
	function()
		local newProfileName = createProfile:GetText()
	
		--Check if there already is a profile with this name
		for k,v in pairs(tProfiles) do
			if(newProfileName == v) then
				return
			end
		end
	
		local tempKey = cPV
		if newProfileName ~= "" then
			YayMounts.db:SetProfile(newProfileName) 

			-- Update the dropdown lists
			YayMounts:AlphaSortProfileDB()
			currentProfile:SetList(tProfiles)
			copyFrom:SetList(tProfiles)
			deleteSelect:SetList(tProfiles)
	  
			-- Select the new profile
			if currentProfile:GetValue() ~= newProfileName then
				for k,v in pairs(tProfiles) do
				  if v == newProfileName then
					tempKey = k
				  end
				end
				currentProfile:SetValue(tempKey)
				copyFrom:SetItemDisabled(tempKey, true)
				copyFrom:SetItemDisabled(cPV, false)
				deleteSelect:SetItemDisabled(tempKey, true)
				deleteSelect:SetItemDisabled(cPV, false)
				cPV = currentProfile:GetValue()
				createProfile:SetText("")
				YayMounts:ReSort()
			end
		end
	end)
  
	currentProfile:SetCallback("OnValueChanged", 
		function() 
			if currentProfile:GetValue() ~= cPV then 
				YayMounts.db:SetProfile(tProfiles[currentProfile:GetValue()])
				copyFrom:SetItemDisabled(currentProfile:GetValue(), true)
				copyFrom:SetItemDisabled(cPV, false)
				deleteSelect:SetItemDisabled(currentProfile:GetValue(), true)
				deleteSelect:SetItemDisabled(cPV, false)
				deleteSelect:SetValue(nil)
				cPV = currentProfile:GetValue() 
			end 
		end)
  
	copyFrom:SetCallback("OnValueChanged", 
		function() 
			copyButton:SetDisabled(false) 
		end)
  
	copyButton:SetCallback("OnClick", 
		function() 
			YayMounts.db:CopyProfile(tProfiles[copyFrom:GetValue()], true) 
			copyFrom:SetValue(nil) 
			copyButton:SetDisabled(true)
			YayMounts:ReSort()
		end)
  
  deleteSelect:SetCallback("OnValueChanged", 
	function()
		if deleteSelect:GetValue() ~= nil then
		  deleteButton:SetDisabled(false) 
		else
		  deleteButton:SetDisabled(true)
		end
	end)
  
  deleteButton:SetCallback("OnClick", 
	function()
		if(copyFrom:GetValue() == deleteSelect:GetValue()) then
			copyFrom:SetValue(nil)
		end
		
		YayMounts.db:DeleteProfile(tProfiles[deleteSelect:GetValue()], true)
		deleteSelect:SetValue(nil) 
		deleteButton:SetDisabled(true)
		YayMounts:AlphaSortProfileDB()
		currentProfile:SetList(tProfiles)
		copyFrom:SetList(tProfiles)
		deleteSelect:SetList(tProfiles)
		deleteSelect:SetItemDisabled(1, true)
		deleteSelect:SetItemDisabled(cPV, true)
	end)
  
  container:AddChild(createProfile)
  container:AddChild(currentProfile)
  container:AddChild(copyFrom)
  container:AddChild(copyButton)
  container:AddChild(deleteSelect)
  container:AddChild(deleteButton)
end

local function SelectGroup(container, event, group)-- Callback function for OnGroupSelected
  container:ReleaseChildren()
  if group == "tab1" then
    DrawGroup1(container)
  elseif group == "tab2" then
    DrawGroup2(container)
  elseif group == "tab3" then
    DrawGroup3(container)
  elseif group == "tab4" then
    DrawGroup4(container)
  end
  currentTab = group
end

function YayMounts:Frames(selectedTab)-- Basic frame
  if not yayFrame then 
    yayFrame = AceGUI:Create("Frame")
    yayFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget); yayFrame = nil end)
    yayFrame:SetTitle("Yay Mounts")
    yayFrame:SetStatusText("Version "..version..", by Cyrae - Windrunner US & Anzu - Kirin Tor US")
    yayFrame:SetLayout("Fill")
    yayFrame:SetWidth(720)
    yayFrame:SetHeight(490)
    
    local tTabs = {
      {text="Mounts ("..self.db.profile.misc.NumMounts["Total"]..")", value="tab1"}, 
      {text="Mount Options", value="tab2"}, 
      {text="Profile Options", value="tab4"},
    }
  
    if class == "MAGE" or class == "PRIEST" or class == "DRUID" or class == "MONK" or class == "ROGUE" then
      tinsert(tTabs, 3, {text=""..localClass.." Options", value = "tab3"})
    end
  
    local tab = AceGUI:Create("TabGroup")
    tab:SetLayout("Flow")
    tab:SetTabs(tTabs)
    tab:SetCallback("OnGroupSelected", SelectGroup)
    tab:SelectTab(selectedTab)
    yayFrame:AddChild(tab)
    
  else
    AceGUI:Release(yayFrame)
    yayFrame, currentTab, currentSubTab = nil
  end
end

function YayMounts:UpdateBindingButtons(btn1, btn2, btn3, btn4)
	btn1:SetKey(GetBindingKey("YAYMOUNTBINDING"))
	btn2:SetKey(GetBindingKey("YAYMOUNTBINDING2"))
	btn3:SetKey(GetBindingKey("YAYMOUNTBINDING3"))
	btn4:SetKey(GetBindingKey("YAYMOUNTBINDING4"))
end

--------------------------------------------------------------
--Mounting and button click functions
--------------------------------------------------------------
function YayMounts:ShouldRepair()
	if(forceRepairMount) then
		return true
	end
	
	local shouldRepair = false

	--Repair mount code
    if size(self.db.profile.misc.RepairMounts) > 0 and self.db.profile.settings["URM"] then
		--Check for low durability equipped gear
		for i = 0, 17 do
			local current, maximum = GetInventoryItemDurability(i)
			if current ~= nil and maximum ~= nil then
				if (current / maximum) <= self.db.profile.misc["MinDurability"] then
				shouldRepair = true
				end
			end
		end
      
		--Check inventory for low durability items
		if(not shouldRepair) then
			for i = 0, 4 do
				for j = 0, GetContainerNumSlots(i) do
					local current, maximum = GetContainerItemDurability(i, j);
					if current ~= nil and maximum ~= nil then
						if (current / maximum) <= self.db.profile.misc["MinDurability"] then
						shouldRepair = true
						end
					end
				end
			end
		end
    end
    
    return shouldRepair
end

function YayMounts:GetRepairMount()
	local i, e = 1, {}
  -- Try and use selected repair mounts first
	for k in pairs(self.db.profile.misc.RepairMounts) do
		if k then
			e[i] = k
			i = i + 1
		end
	end

	-- Use any repair mount if none selected
	if #e == 0 then 
		for k in pairs(self.db.profile.misc.RepairMounts) do
			e[i] = k
			i = i + 1
		end
	end
	
	return e
end

---- Helper method to get the passenger mounts
function YayMounts:BuildPassengerTable(mountType)
	local i, e = 1, {}
  
	for k,v in pairs(YayMountsType["Passenger"]) do
		local mountIndex = self.db.profile.mounts[k][3]
		if YayMounts:IsUsableMount(mountIndex) and v == mountType then
			e[i] = k
			i = i + 1
		end
	end
  
  -- If we wanted flying mounts, but have none, use any passenger mount
	if #e == 0 then
		for k,v in pairs(YayMountsType["Passenger"]) do
			local mountIndex = self.db.profile.mounts[k][3]
			if YayMounts:IsUsableMount(mountIndex) then
				e[i] = k
				i = i + 1
			end
		end
	end

	return e
end

function YayMounts:BuildStandardMountTable(mountType)
	local i, e = 1, {}
	local forceFlyingMounts = self.db.profile.settings["FFM"] and mountType == "ground" and IsUsableSpell(59976)
  
	for k,v in pairs(self.db.profile.mounts) do  
		local isCorrectType = forceFlyingMounts or v[2] == mountType
		if YayMounts:IsUsableMount(v[3]) and v[1] and isCorrectType then
			e[i] = k
			i = i + 1
		end
	end
  
	return e
end

function YayMounts:Mounting(flyable, zone)
	if not self.db.profile.settings["HM"] then
		return nil
	end
		
	local e = {}
	local useRepairMount = 1, false
	local mountType
	
	if flyable then
		mountType = "fly"
	else
		mountType = "ground"
	end
	
	--Repair mount
	if YayMounts:ShouldRepair() then
		e = YayMounts:GetRepairMount()
	elseif usePassengerMount then
		e = YayMounts:BuildPassengerTable(mountType)
	else
		e = YayMounts:BuildStandardMountTable(mountType)
	end
	
	-- Filter out mounts for certain zones (AQ, etc)
	e = YayMounts:Filter(e, zone)
	
	-- Load emergency mounts if we have no usable mounts yet
	if #e == 0 and self.db.profile.settings["ER"] then
		if(mountType == "fly" and self.db.profile.misc.NumMounts.Flying == 0) then
			mountType = "ground"
		end
		
		local i = 1
		for spellID,v in pairs(self.db.profile.mounts) do
			if YayMounts:IsUsableMount(v[3]) and v[2] == mountType then
				e[i] = spellID
				i = i + 1
			end
		end
	end
	
	-- Final check to remove any currently unusable mounts
	for k,spellId in pairs(e) do
		if not YayMounts:IsUsableMount(self.db.profile.mounts[spellId][3]) then
			tremove(e, k)
		end
	end
	
	if #e == 0 then
		self:Print(YayMountsLocale.String.NoMounts)
		return nil
	end
	
	return YayMounts:SpellToName(e[math.random(#e)])
end

function YayMounts:PreClick()
	local _,_,WGActive,_,_,_ = GetWorldPVPAreaInfo(1)
	local isFlyable
	local broom = false
	local zone = GetRealZoneText()
	local loanedMount = YayMounts:CheckLoanerMount()

	-- Check if we can fly
	isFlyable = IsFlyableArea()
	
	--Check if we're in Draenor for flying(remove this in 6.1!)
	if(GetCurrentMapContinent() == 7 or zone == "Tanaan Jungle") then
		isFlyable = false
		if(zone == "Nagrand" and self.db.profile.settings["TELAARI"] == true) then
			if(YayMounts:CheckNagrandSpecialMount()) then --Telaari Talbuk/Horde wolf
				return
			end
		end
	end

	--Check if in Wintergrasp
	if zone == YayMountsLocale.Zone.Wintergrasp then 
		isFlyable = not WGActive
	end
     
	-- check if we can use flying broom
	if GetItemCount(37011, false) > 0 and not playerCombat and self.db.profile.settings["FBP"] == true then
		broom = true
	end
	
  ------------Dismounting logic-------------
	if IsMounted() and IsFlying() then
		if class == "DRUID" and self.db.profile.settings["MTFF"] and GetUnitSpeed("player") ~= 0 then 
			YayMounts:Druids(isFlyable, zone) -- Druid: Flying mount to flight form   
		elseif self.db.profile.settings["NDWF"] then
			YayMounts:SetMacroButton()
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
  
	elseif IsMounted() then
		if IsSwimming() and class == "DRUID" and IsUsableSpell(783) and GetUnitSpeed("player") ~= 0  then
			YayMounts:SetMacroButton(YayMounts:SpellToName(783)) -- Aquatic Form  
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
    
	elseif CanExitVehicle() then
		YayMounts:SetMacroButton()
		VehicleExit()
    
	------------Not currently mounted-------------
	elseif YayMounts:PriestOrMageSpecial() then
		return -- Going to use slowfall/levitate
    
	elseif isFlyable and IsOutdoors() then
		if broom then --flying broom
			YayMounts:SetMacroButton(YayMounts:ItemToName(37011))
		elseif class == "SHAMAN" and IsUsableSpell(2645) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		elseif class == "MONK" and self.db.profile.settings["MR"] and IsUsableSpell(109132) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(109132)) --Roll
		elseif class == "ROGUE" and self.db.profile.settings["RS"] and IsUsableSpell(2983) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2983)) --Sprint    
		elseif IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif class == "DRUID" then
			YayMounts:Druids(isFlyable, zone)
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone))
		end
    
	elseif not isFlyable and IsOutdoors() and zone ~= YayMountsLocale.Zone.VortexPinnacle then
		if zone == YayMountsLocale.Zone.Oculus and YayMounts:Filter(nil, zone) then
			return
		elseif broom then --flying broom
			YayMounts:SetMacroButton(YayMounts:ItemToName(37011))     
		elseif class == "SHAMAN" and IsUsableSpell(2645) and (playerCombat or GetUnitSpeed("player") ~= 0 or not self.db.profile.settings["HM"]) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		elseif class == "MONK" and self.db.profile.settings["MR"] and IsUsableSpell(109132) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(109132)) --Roll
		elseif class == "ROGUE" and self.db.profile.settings["RS"] and IsUsableSpell(2983) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2983)) --Sprint
		elseif IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif loanedMount ~= nil and not playerCombat then
			YayMounts:SetMacroButton(YayMounts:ItemToName(loanedMount))
		elseif class == "DRUID" then
			YayMounts:Druids(isFlyable, zone)
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone))
		end
    
	elseif not IsOutdoors() or zone == YayMountsLocale.Zone.VortexPinnacle then
		if IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif class == "DRUID" and IsUsableSpell(768) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(768)) -- Cat form
		elseif class == "SHAMAN" and IsUsableSpell(2645) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		elseif class == "MONK" and self.db.profile.settings["MR"] and IsUsableSpell(109132) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(109132)) --Roll
		elseif class == "ROGUE" and self.db.profile.settings["RS"] and IsUsableSpell(2983) and (playerCombat or GetUnitSpeed("player") ~= 0) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2983)) --Sprint
		end
    
	else
		--No logic left, just try to use a mount
		YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone)) 
	end
end

function YayMounts:ForcePassengerMount()
	usePassengerMount = true
	YayMounts:PreClick()
	usePassengerMount = false
end

function YayMounts:ForceRepairMount()
	forceRepairMount = true
	YayMounts:PreClick()
	forceRepairMount = false
end

-- Tries to use the Nagrand outpost special ability
function YayMounts:CheckNagrandSpecialMount()
	local garrisonAbilityName = GetSpellInfo(161691)
	local _,_,_,_,_,_,spellID = GetSpellInfo(garrisonAbilityName)
	if(spellID == 165803 or spellID == 164222) then -- Have the proper outpost
		if(playerCombat) then
			YayMounts:SetMacroButtonCombat(garrisonAbilityName)
		else
			YayMounts:SetMacroButton(garrisonAbilityName)
		end
		return true
	else
		return false
	end
end

-- Checks if slowfall or levitate should be used
function YayMounts:PriestOrMageSpecial()
  if not playerCombat then
    if (IsFalling() or GetUnitSpeed("player") ~= 0) and class == "PRIEST" and IsUsableSpell(1706) and self.db.profile.settings["PMLS"] then --IsUsableSpell checks for reagents/glyphs
      YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(1706).."\n/cancelaura "..YayMounts:SpellToName(1706)) --Levitate
      return true
      
    elseif (IsFalling() or GetUnitSpeed("player") ~= 0) and class == "MAGE" and IsUsableSpell(130) and self.db.profile.settings["PMLS"] then
      YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(130).."\n/cancelaura "..YayMounts:SpellToName(130)) --Slow fall
      return true
      
    else
      return false
    end
  end
end

function YayMounts:Druids(flyable, zone)
	local travelForm, flightForm = 783, 783 -- 3 in 1 travel form
	
	for glyphIndex = 3, 6, 1 do
		local _, _, _, glyphSpell, _ = GetGlyphSocketInfo(glyphIndex)
		if(glyphSpell == 114338) then -- Glyph of the Stag
			flightForm = 165962 --Flight form is separate with Stag Glyph
		end
	end

	if GetUnitSpeed("player") ~= 0 then
		if flyable then
			YayMounts:SetMacroButton(YayMounts:SpellToName(flightForm))
		else
			YayMounts:SetMacroButton(YayMounts:SpellToName(travelForm))
		end
		
	elseif(flyable and not playerCombat) then
		if self.db.profile.settings["FFP"] or IsFalling() or IsFlying() or GetShapeshiftForm() == 3 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(flightForm))
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone), 1)
		end
    
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(false, zone), 1)  
	end
end

function YayMounts:Swimming(flyable, zone, broom)
	local aquaticForm = 783

	local timer, _,_,scale,_,_ = GetMirrorTimerInfo(2)
	local name, _, _, _, _, _, _, _, _, _, _ = UnitBuff("player", YayMountsLocale.Misc.WaterBreathing)
	if timer == "UNKNOWN" or scale >= 0 then
		scale = 1
	end
  
	if flyable and not playerCombat then
	if class == "DRUID" and IsUsableSpell(aquaticForm) and GetUnitSpeed("player") ~= 0 then
		YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) --Aquatic Form
	elseif class == "DRUID" then
		YayMounts:Druids(flyable, zone)
		--elseif broom then
		--YayMounts:SetMacroButton(YayMounts:ItemToName(33176)) --Flying Broom
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(true, zone))
	end
    
	elseif IsOutdoors() and not playerCombat then
		if class == "DRUID" and IsUsableSpell(aquaticForm) and GetUnitSpeed("player") ~= 0 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) --Aquatic Form
		else 
			if zone == YayMountsLocale.Zone.Vashjir1 or zone == YayMountsLocale.Zone.Vashjir2 or zone == YayMountsLocale.Zone.Vashjir3 or zone == YayMountsLocale.Zone.Vashjir4 or zone == YayMountsLocale.Zone.Vashjir5 and self.db.profile.mounts[75207][1] == true then --Abyssal Seahorse
				YayMounts:SetMacroButton(YayMounts:SpellToName(75207)) --Abyssal Seahorse
			elseif IsUsableSpell(98718) and self.db.profile.mounts[98718][1] == true then
				YayMounts:SetMacroButton(YayMounts:SpellToName(98718)) --Subdued Seahorse
			elseif IsUsableSpell(64731) and self.db.profile.mounts[64731][1] then
				YayMounts:SetMacroButton(YayMounts:SpellToName(64731)) --Sea Turtle
			else
				YayMounts:SetMacroButton(YayMounts:Mounting(false, zone))
			end
		end
    
	elseif not playerCombat then
		if class == "DRUID" and IsUsableSpell(aquaticForm) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) -- Aquatic Form
		elseif class == "SHAMAN" and IsUsableSpell(2645) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		end
	end
end

--check if we have cold weather flying, a loaned mount, and/or flight master's license
function YayMounts:CheckLoanerMount()
  local zone = GetRealZoneText()
  if zone == YayMountsLocale.Zone.StormPeaks or zone == YayMountsLocale.Zone.Icecrown then
    if GetItemCount(44221, false) > 0 then
      return 44221
    elseif GetItemCount(44229, false) > 0 then
      return 44229
    else
      return nil
    end
  end
  return nil
end

function YayMounts:SetMacroButton(mount, button)
	if not playerCombat then
		if mount and button == 2 then
			YayMountsButton2:SetAttribute("macrotext", "/use "..YayMounts:Mounting(false, GetRealZoneText()))
		elseif mount and usePassengerMount then
			YayMountsButton3:SetAttribute("macrotext", "/use "..mount)
		elseif mount and forceRepairMount then
			YayMountsButton4:SetAttribute("macrotext", "/use "..mount)
		elseif mount then
			YayMountsButton:SetAttribute("macrotext", "/use "..mount)   
		else
			YayMountsButton:SetAttribute("macrotext", nil)
		end
	end
end

function YayMounts:SetMacroButtonCombat(mount, button)
	if mount and button == 2 then
		YayMountsButton2:SetAttribute("macrotext", "/use "..YayMounts:Mounting(false, GetRealZoneText()))
	elseif mount and button == 3 then
		YayMountsButton3:SetAttribute("macrotext", "/use "..mount)
	elseif mount then
		YayMountsButton:SetAttribute("macrotext", "/use "..mount)
	else
		YayMountsButton:SetAttribute("macrotext", nil)
    end
end

function YayMounts:Filter(e, zone)
  if zone ~= YayMountsLocale.Zone.AQ40 and e ~= nil then
    for k,v in pairs(e) do
      if v == YayMountsType["Ahn'Qiraj"][v] then
        tremove(e, k)
      end
    end
    return e
    
  elseif zone == YayMountsLocale.Zone.AQ40 and e ~= nil then
    local aq40 = false
    local aq40Table = {}
    local i = 1
    
    for k,v in pairs(e) do
      if v == YayMountsType["Ahn'Qiraj"][v] then
        aq40 = true
        
        break
      end
    end
    
    if aq40 then
      for k,v in pairs(e) do
        if v == YayMountsType["Ahn'Qiraj"][v] then
          aq40Table[i] = v
          i = i + 1
        end
      end
      return aq40Table
    
    else
      return e
    end
    
  
  elseif zone == YayMountsLocale.Zone.Oculus and e == nil then
    if GetItemCount(37859) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37859))
      return true
    elseif GetItemCount(37860) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37860))
      return true
    elseif GetItemCount(37815) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37815))
      return true
    end
    return false
    
  else
    return e
  end
end

function YayMounts:EventHandler(arg1, arg2)
  if  arg1 == "PLAYER_REGEN_DISABLED" then
    playerCombat = true
    if class == "SHAMAN" then
      YayMounts:SetMacroButtonCombat(YayMounts:SpellToName(2645))
    elseif class == "DRUID" then
      if not self.db.profile.settings["TTT"] then
        YayMounts:SetMacroButtonCombat("[swimming] "..YayMounts:SpellToName(783).."; [outdoors] "..YayMounts:SpellToName(783).."; "..YayMounts:SpellToName(768))
      else
        YayMounts:SetMacroButtonCombat("[swimming] !"..YayMounts:SpellToName(783).."; [outdoors] !"..YayMounts:SpellToName(783).."; !"..YayMounts:SpellToName(768))
      end
      
    elseif class == "PRIEST" then
      if IsUsableSpell(1706) and self.db.profile.settings["PMLS"] then --IsUsableSpell checks for reagents/glyphs
        YayMounts:SetMacroButtonCombat("[@player] "..YayMounts:SpellToName(1706).."\n/cancelaura "..YayMounts:SpellToName(1706)) --Levitate
      end
      
    elseif class == "MAGE" then
      if IsUsableSpell(130) and self.db.profile.settings["PMLS"] then
        YayMounts:SetMacroButtonCombat("[@player] "..YayMounts:SpellToName(130).."\n/cancelaura "..YayMounts:SpellToName(130)) --Slow fall
      end
    
    elseif class == "MONK" and self.db.profile.settings["MR"] then
      if IsUsableSpell(109132) then
        YayMounts:SetMacroButtonCombat(YayMounts:SpellToName(109132)) --Roll
      end
    end
    
  elseif arg1 == "PLAYER_REGEN_ENABLED" then
    playerCombat = false
    YayMounts:SetMacroButton()
    
  elseif arg1 == "PLAYER_LEVEL_UP" then
    if arg2 == 20 then
      YayMounts:ReSort()
      self:UnregisterEvent("PLAYER_LEVEL_UP")
    end
    
  elseif arg1 == "COMPANION_LEARNED" then
    YayMounts:ReSort()
    if yayFrame then
      local ct, cst = currentTab, currentSubTab
      YayMounts:Frames()
      currentSubTab = cst
      YayMounts:Frames(ct)
    end
    
  elseif arg1 == "PLAYER_ENTERING_WORLD" then
    local faction, _ = UnitFactionGroup("player")
    if(faction == "Alliance") then
      playerFaction = 1
    elseif(faction == "Horde") then
      playerFaction = 0
    else
      self:Print("Unrecognized faction: ", faction)
      playerFaction = -1
    end
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
  end
end
--------------------------------------------------------------
--Keybinding stuff
--------------------------------------------------------------
function YayMounts:MapOverrides()
  if not InCombatLockdown() then
    ClearOverrideBindings(YayMountsButton)
    local key1 = GetBindingKey("YAYMOUNTBINDING")
    if key1 then
      SetOverrideBindingClick(YayMountsButton, true, key1, YayMountsButton:GetName())
    end

    ClearOverrideBindings(YayMountsButton2)
    key1 = GetBindingKey("YAYMOUNTBINDING2")
    if key1 then
      SetOverrideBindingClick(YayMountsButton2, true, key1, YayMountsButton2:GetName())
    end
    
    ClearOverrideBindings(YayMountsButton3)
    key1 = GetBindingKey("YAYMOUNTBINDING3")
    if key1 then
      SetOverrideBindingClick(YayMountsButton3, true, key1, YayMountsButton3:GetName())
    end
	
	ClearOverrideBindings(YayMountsButton4)
    key1 = GetBindingKey("YAYMOUNTBINDING4")
    if key1 then
      SetOverrideBindingClick(YayMountsButton4, true, key1, YayMountsButton4:GetName())
    end
  end
end

function YayMounts:Keybinding(key, buttonID)
	if not key then
		key = ""
	end

	if not InCombatLockdown() then
		if buttonID == 1 then
			local key1 = GetBindingKey("YAYMOUNTBINDING")
			if key1 then
				SetBinding(key1)
			end
			SetBinding(key, "YAYMOUNTBINDING")
		  
		elseif buttonID == 2 then
			local key1 = GetBindingKey("YAYMOUNTBINDING2")
			if key1 then
				SetBinding(key1)
			end
			SetBinding(key, "YAYMOUNTBINDING2")
		
		elseif buttonID == 3 then
			local key1 = GetBindingKey("YAYMOUNTBINDING3")
			if key1 then
				SetBinding(key1)
			end
			SetBinding(key, "YAYMOUNTBINDING3")
		
		elseif buttonID == 4 then
			local key1 = GetBindingKey("YAYMOUNTBINDING4")
			if key1 then
				SetBinding(key1)
			end
			SetBinding(key, "YAYMOUNTBINDING4")
		end
	end
  
	SaveBindings(GetCurrentBindingSet())
end